@@ -158,6 +158,8 @@ fs_codec_copy (const FsCodec * codec)
   copy->media_type = codec->media_type;
   copy->clock_rate = codec->clock_rate;
   copy->channels = codec->channels;
+  copy->ABI.ABI.maxptime = codec->ABI.ABI.maxptime;
+  copy->ABI.ABI.ptime = codec->ABI.ABI.ptime;
 
   copy->encoding_name = g_strdup (codec->encoding_name);
 
@@ -329,7 +331,7 @@ fs_codec_list_from_keyfile (const gchar *filename, GError **error)
     }
 
     for (j = 0; j < keys_count && keys[j]; j++) {
-      if (!strcmp ("clock-rate", keys[j])) {
+      if (!g_ascii_strcasecmp ("clock-rate", keys[j])) {
         codec->clock_rate = g_key_file_get_integer (keyfile, groups[i], keys[j],
             &gerror);
         if (gerror) {
@@ -337,7 +339,7 @@ fs_codec_list_from_keyfile (const gchar *filename, GError **error)
           goto keyerror;
         }
 
-      } else if (!strcmp ("id", keys[j])) {
+      } else if (!g_ascii_strcasecmp ("id", keys[j])) {
          codec->id = g_key_file_get_integer (keyfile, groups[i], keys[j],
             &gerror);
         if (gerror) {
@@ -348,7 +350,7 @@ fs_codec_list_from_keyfile (const gchar *filename, GError **error)
         if (codec->id < 0)
           codec->id = FS_CODEC_ID_DISABLE;
 
-      } else if (!strcmp ("channels", keys[j])) {
+      } else if (!g_ascii_strcasecmp ("channels", keys[j])) {
          codec->channels = g_key_file_get_integer (keyfile, groups[i], keys[j],
             &gerror);
         if (gerror) {
@@ -356,6 +358,20 @@ fs_codec_list_from_keyfile (const gchar *filename, GError **error)
           goto keyerror;
         }
 
+      } else if (!g_ascii_strcasecmp ("maxptime", keys[j])) {
+        codec->ABI.ABI.maxptime = g_key_file_get_integer (keyfile, groups[i],
+            keys[j], &gerror);
+        if (gerror) {
+          codec->ABI.ABI.maxptime = 0;
+          goto keyerror;
+        }
+      } else if (!g_ascii_strcasecmp ("ptime", keys[j])) {
+        codec->ABI.ABI.ptime = g_key_file_get_integer (keyfile, groups[i],
+            keys[j], &gerror);
+        if (gerror) {
+          codec->ABI.ABI.ptime = 0;
+          goto keyerror;
+        }
       } else {
         FsCodecParameter *param = g_slice_new (FsCodecParameter);
 
@@ -450,6 +466,12 @@ fs_codec_to_string (const FsCodec *codec)
       codec->id, fs_media_type_to_string (codec->media_type),
       codec->encoding_name, codec->clock_rate, codec->channels);
 
+  if (codec->ABI.ABI.maxptime)
+    g_string_append_printf (string, " maxptime=%u", codec->ABI.ABI.maxptime);
+
+  if (codec->ABI.ABI.ptime)
+    g_string_append_printf (string, " ptime=%u", codec->ABI.ABI.ptime);
+
   for (item = codec->optional_params;
        item;
        item = g_list_next (item)) {
@@ -485,7 +507,7 @@ compare_lists (GList *list1, GList *list2)
          item2 = g_list_next (item2)) {
       FsCodecParameter *param2 = item2->data;
 
-      if (!strcmp (param1->name, param2->name) &&
+      if (!g_ascii_strcasecmp (param1->name, param2->name) &&
           !strcmp (param1->value, param2->value))
         break;
     }
@@ -522,9 +544,11 @@ fs_codec_are_equal (const FsCodec *codec1, const FsCodec *codec2)
       codec1->media_type != codec2->media_type ||
       codec1->clock_rate != codec2->clock_rate ||
       codec1->channels != codec2->channels ||
+      codec1->ABI.ABI.maxptime != codec2->ABI.ABI.maxptime ||
+      codec1->ABI.ABI.ptime != codec2->ABI.ABI.ptime ||
       codec1->encoding_name == NULL ||
       codec2->encoding_name == NULL ||
-      strcmp (codec1->encoding_name, codec2->encoding_name))
+      g_ascii_strcasecmp (codec1->encoding_name, codec2->encoding_name))
     return FALSE;
 
 
