@@ -67,15 +67,12 @@ G_DEFINE_TYPE(FsElementAddedNotifier, fs_element_added_notifier, G_TYPE_OBJECT);
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-static GObjectClass *parent_class = NULL;
-
 static void
 fs_element_added_notifier_class_init (FsElementAddedNotifierClass *klass)
 {
   GObjectClass *gobject_class;
 
   gobject_class = (GObjectClass *) klass;
-  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = fs_element_added_notifier_finalize;
 
@@ -162,7 +159,6 @@ static void
 _bin_unparented_cb (GstObject *object, GstObject *parent, gpointer user_data)
 {
   GstIterator *iter = NULL;
-  FsElementAddedNotifier *notifier = FS_ELEMENT_ADDED_NOTIFIER (user_data);
   gboolean done;
 
   /* Return if there was no handler connected */
@@ -181,6 +177,7 @@ _bin_unparented_cb (GstObject *object, GstObject *parent, gpointer user_data)
       case GST_ITERATOR_OK:
         if (GST_IS_BIN (item))
           _bin_unparented_cb (GST_OBJECT (item), object, user_data);
+        gst_object_unref (item);
         break;
       case GST_ITERATOR_RESYNC:
         // We don't rollback anything, we just ignore already processed ones
@@ -197,10 +194,6 @@ _bin_unparented_cb (GstObject *object, GstObject *parent, gpointer user_data)
   }
 
   gst_iterator_free (iter);
-
-  g_object_weak_unref (G_OBJECT (object), (GWeakNotify) g_object_unref,
-      notifier);
-  g_object_unref (notifier);
 }
 
 
@@ -272,104 +265,34 @@ _bin_added_from_keyfile (FsElementAddedNotifier *notifier, GstBin *bin,
   for (i = 0; keys[i]; i++)
   {
     GParamSpec *param_spec;
-    GValue key_value = { 0 };
     GValue prop_value = { 0 };
-
-    gchar *str_key_value;
-    gboolean bool_key_value;
-    gint int_key_value;
-    gdouble double_key_value;
-    glong long_key_value;
-    gulong ulong_key_value;
+    gchar *str_value;
 
     DEBUG ("getting %s", keys[i]);
-    param_spec = g_object_class_find_property
-      (G_OBJECT_GET_CLASS(element), keys[i]);
+    param_spec = g_object_class_find_property (G_OBJECT_GET_CLASS(element),
+        keys[i]);
 
-    /* If the paremeter does not exist, or is one of those,
-     * then lets skip it
-     * TODO: What if we want to pass GstCaps as strings?
-     */
-    if (!param_spec ||
-        g_type_is_a (param_spec->value_type, G_TYPE_OBJECT) ||
-        g_type_is_a (param_spec->value_type, GST_TYPE_MINI_OBJECT) ||
-        g_type_is_a (param_spec->value_type, G_TYPE_INTERFACE) ||
-        g_type_is_a (param_spec->value_type, G_TYPE_BOXED) ||
-        g_type_is_a (param_spec->value_type, G_TYPE_GTYPE) ||
-        g_type_is_a (param_spec->value_type, G_TYPE_POINTER))
+    if (!param_spec)
     {
+      DEBUG ("Property %s does not exist in element %s, ignoring",
+          keys[i], name);
       continue;
     }
 
     g_value_init (&prop_value, param_spec->value_type);
 
-    switch (param_spec->value_type)
+    str_value = g_key_file_get_value (keyfile, name, keys[i], NULL);
+    if (str_value && gst_value_deserialize (&prop_value, str_value))
     {
-      case G_TYPE_STRING:
-        str_key_value = g_key_file_get_value (keyfile, name,
-            keys[i], NULL);
-        g_value_init (&key_value, G_TYPE_STRING);
-        g_value_set_string (&key_value, str_key_value);
-        DEBUG ("%s is a string: %s", keys[i], str_key_value);
-        g_free (str_key_value);
-        break;
-      case G_TYPE_BOOLEAN:
-        bool_key_value = g_key_file_get_boolean (keyfile, name,
-            keys[i], NULL);
-        g_value_init (&key_value, G_TYPE_BOOLEAN);
-        g_value_set_boolean (&key_value, bool_key_value);
-        DEBUG ("%s is a boolean: %d", keys[i], bool_key_value);
-        break;
-      case G_TYPE_UINT64:
-      case G_TYPE_INT64:
-      case G_TYPE_DOUBLE:
-        double_key_value = g_key_file_get_double (keyfile, name,
-            keys[i], NULL);
-        g_value_init (&key_value, G_TYPE_DOUBLE);
-        g_value_set_double (&key_value, double_key_value);
-        DEBUG ("%s is a uint64", keys[i]);
-        DEBUG ("%s is a int64", keys[i]);
-        DEBUG ("%s is a double: %f", keys[i], double_key_value);
-        break;
-      case G_TYPE_ULONG:
-        str_key_value = g_key_file_get_value (keyfile, name, keys[i],
-            NULL);
-        ulong_key_value = strtoul (str_key_value, NULL, 10);
-        g_value_init (&key_value, G_TYPE_ULONG);
-        g_value_set_ulong (&key_value, ulong_key_value);
-        DEBUG ("%s is a ulong: %lu", keys[i], ulong_key_value);
-        break;
-      case G_TYPE_LONG:
-        str_key_value = g_key_file_get_value (keyfile, name, keys[i],
-            NULL);
-        long_key_value = strtol (str_key_value, NULL, 10);
-        g_value_init (&key_value, G_TYPE_LONG);
-        g_value_set_long (&key_value, long_key_value);
-        DEBUG ("%s is a long: %ld", keys[i], long_key_value);
-        break;
-      case G_TYPE_INT:
-      case G_TYPE_UINT:
-      case G_TYPE_ENUM:
-      default:
-        int_key_value = g_key_file_get_integer (keyfile, name,
-            keys[i], NULL);
-        g_value_init (&key_value, G_TYPE_INT);
-        g_value_set_int (&key_value, int_key_value);
-        DEBUG ("%s is a int: %d", keys[i], int_key_value);
-        DEBUG ("%s is a uint", keys[i]);
-        DEBUG ("%s is an enum", keys[i]);
-        DEBUG ("%s is something else, attempting to int conv", keys[i]);
-        break;
+      DEBUG ("Setting %s to on %s", keys[i], name);
+      g_object_set_property (G_OBJECT (element), keys[i], &prop_value);
     }
-
-    if (!g_value_transform (&key_value, &prop_value))
+    else
     {
-      DEBUG ("Could not transform gvalue pair");
-      continue;
+      DEBUG ("Could not read value for property %s", keys[i]);
     }
-
-    DEBUG ("Setting %s to on %s", keys[i], name);
-    g_object_set_property (G_OBJECT (element), keys[i], &prop_value);
+    g_free (str_value);
+    g_value_unset (&prop_value);
   }
 
   g_strfreev (keys);
@@ -402,6 +325,36 @@ fs_element_added_notifier_set_properties_from_keyfile (
 }
 
 
+/**
+ * fs_element_added_notifier_set_properties_from_file:
+ * @notifier: a #FsElementAddedNotifier
+ * @filename: The name of the keyfile to use
+ * @error: location of a #GError, or %NULL if no error occured
+ *
+ * Same as fs_element_added_notifier_set_properties_from_keyfile() but using
+ * the name of the file to load instead of the #GKeyFile directly.
+ *
+ * Returns: %TRUE if the file was successfully loaded, %FALSE otherwise
+ */
+gboolean
+fs_element_added_notifier_set_properties_from_file (
+    FsElementAddedNotifier *notifier,
+    const gchar *filename,
+    GError **error)
+{
+  GKeyFile *keyfile = g_key_file_new ();
+
+  if (!g_key_file_load_from_file (keyfile, filename, G_KEY_FILE_NONE, error))
+  {
+    g_key_file_free (keyfile);
+    return FALSE;
+  }
+
+  fs_element_added_notifier_set_properties_from_keyfile(notifier, keyfile);
+
+  return TRUE;
+}
+
 static void
 _element_added_callback (GstBin *parent, GstElement *element,
     gpointer user_data)
@@ -412,15 +365,12 @@ _element_added_callback (GstBin *parent, GstElement *element,
     GstIterator *iter = NULL;
     gboolean done;
 
-    g_object_ref (notifier);
-    g_object_weak_ref (G_OBJECT (element), (GWeakNotify) g_object_unref,
-        notifier);
-    g_signal_connect (element, "element-added",
-        G_CALLBACK (_element_added_callback), notifier);
+    g_signal_connect_object (element, "element-added",
+        G_CALLBACK (_element_added_callback), notifier, 0);
 
     if (parent)
-      g_signal_connect (element, "parent-unset",
-          G_CALLBACK (_bin_unparented_cb), notifier);
+      g_signal_connect_object (element, "parent-unset",
+          G_CALLBACK (_bin_unparented_cb), notifier, 0);
 
     iter = gst_bin_iterate_elements (GST_BIN (element));
 
