@@ -142,6 +142,8 @@ _new_local_candidate (FsStream *stream, FsCandidate *candidate)
   if (candidate->component_id == FS_COMPONENT_RTCP && no_rtcp)
     return;
 
+  st->got_candidates = TRUE;
+
   GST_DEBUG ("%d:%d: Setting remote candidate for component %d",
       other_st->dat->id,
       other_st->target->id,
@@ -175,6 +177,20 @@ _current_send_codec_changed (FsSession *session, FsCodec *codec)
   g_free (str);
 }
 
+static void
+_local_candidates_prepared (FsStream *stream)
+{
+  struct SimpleTestStream *st = g_object_get_data (G_OBJECT (stream),
+      "SimpleTestStream");
+
+  if (!st->got_candidates)
+  {
+    g_debug ("Skipping test because there are no candidates");
+    g_main_loop_quit (loop);
+  }
+
+}
+
 
 static gboolean
 _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
@@ -296,7 +312,8 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
           ts_fail_unless (
               gst_implements_interface_check (GST_MESSAGE_SRC (message),
                   FS_TYPE_CONFERENCE),
-              "Received farsight-error from non-farsight element");
+              "Received farsight-current-send-codec-change from non-farsight"
+              " element");
 
           ts_fail_unless (
               gst_structure_has_field_typed (s, "session", FS_TYPE_SESSION),
@@ -318,6 +335,28 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
 
           _current_send_codec_changed (session, codec);
         }
+        else if (gst_structure_has_name (s,
+                "farsight-local-candidates-prepared"))
+        {
+          FsStream *stream;
+          const GValue *value;
+
+          ts_fail_unless (
+              gst_implements_interface_check (GST_MESSAGE_SRC (message),
+                  FS_TYPE_CONFERENCE),
+              "Received farsight-local-candidates-prepared from non-farsight"
+              " element");
+
+          ts_fail_unless (
+              gst_structure_has_field_typed (s, "stream", FS_TYPE_STREAM),
+              "farsight-local-candidates-prepared structure"
+              " has no stream field");
+
+          value = gst_structure_get_value (s, "stream");
+          stream = g_value_get_object (value);
+
+          _local_candidates_prepared (stream);
+        }
 
        }
       break;
@@ -1450,6 +1489,7 @@ static void unref_stream_init (void)
   {
     GstBus *bus = gst_pipeline_get_bus (GST_PIPELINE (dats[i]->pipeline));
 
+    gst_bus_set_sync_handler (bus, NULL, NULL);
     gst_bus_set_sync_handler (bus, unref_stream_sync_handler, dats[i]);
     gst_object_unref (bus);
   }
